#include "Demon.h"

Demon::Demon(int eventId, int spriteIndex, Path& path):
    id(eventId),
    sprite("resources/demons.png", 32, 32, -1, 0, 0, 0, false),
    path(path),
    frame(0)
{
    lifeTimer = 60 * 30;
    percent = 0;
    wasHit = false;
    wasMiss = false;
    sprite.setFrame(spriteIndex);

    sprite.setFlipped(spriteIndex < 6);
    scale = 1.4f;
    alpha = 1.f;
    x = y = -1000;
    frame = rand() % 1000;
}

Demon::~Demon()
{
    //dtor
}

void Demon::hit()
{
    if(wasHit || wasMiss)
    {
        return;
    }
    wasHit = true;
    //left to right
    dx = 3 + (rand() / (float)RAND_MAX);
    dy = 3 * (rand()/(float)RAND_MAX - 0.5f);
    if(path.getPosition(0).x < path.getPosition(1.f).x)
    {
        dx = -dx;
    }
    lifeTimer = 120;

}

void Demon::miss()
{
    if(wasHit || wasMiss)
    {
        return;
    }
    wasMiss = true;
    lifeTimer = 30;
    dx = -3;
    dy = -1;
    if(path.getPosition(0).x < path.getPosition(1.f).x)
    {
        dx = -dx;
    }
}
void Demon::setPercent(float p)
{
    if(p > 1.f)
    {
        return;
    }
    if(p < 0)
    {
        return;
    }
    if(wasHit || wasMiss)
    {
        return;
    }
    this->percent = p;
    sf::Vector2f pos = path.getPosition(this->percent);
    dx = pos.x - x;
    dy = pos.y - y;
    x = pos.x;
    y = pos.y;

    if(fabs(x) > 1000.f)
    {
        printf("!! %f - %f, %f\n", this->percent, x, y);
    }
}
void Demon::step()
{
    frame++;


    lifeTimer--;
    if(wasHit || wasMiss)
    {
        scale += 0.014f;
        alpha *= 0.96f;
        x += dx;
        y += dy;
        if(wasMiss)
        {
            alpha *= 0.93f;
        }
    }

}

void Demon::draw(sf::RenderWindow& window, int xOffset, int missTimer)
{
    if(wasHit)
    {
        //scale  = 5.f;
    }
    sprite.setScale(scale);
    sprite.setColor(sf::Color(255, 255, 255, alpha * 255));
    sprite.setRotation(cosf(percent / 3.f + frame / 41.f) * 45);
    sprite.setPosition(x + xOffset, y);
    sprite.draw(window, ResourceManager::shader);
    if(missTimer)
    {
        sprite.setColor(sf::Color(255, 255, 255, alpha * 150));
        sprite.setScale(scale + (missTimer / 13.f));
        sprite.draw(window);
    }
}
