#include "EText.h"
#include "ResourceManager.h"

bool EText::initialized = false;
sf::Texture EText::fontTexture;
const int EText::CHAR_ANIMATE_FRAMES = 6;

EText::EText(string s, float x, float y):
    x(x),
    y(y),
    frame(0),
    text(s),
    sprite(*ResourceManager::getTexture("font.png")),
    scale(1.f),
    shake(false),
    animateIn(false),
    doneAnimating(false),
    wave(false),
    animateSpeed(CHAR_ANIMATE_FRAMES),
    animateTimer(0),
    animateChar(0)
{

}

EText::~EText()
{
    //dtor
}

void EText::draw(sf::RenderWindow& window)
{
    frame++;
    ///Distinct from the subframe dimensions
    static const float charWidth = 13;
    static const float charHeight = 30;
    int line = 0;
    int currCharacter = 0;

    if(animateIn && !doneAnimating)
    {
        animateTimer++;
        if(animateTimer == animateSpeed)
        {
            animateTimer = 0;
            animateChar++;
            if(animateChar >= (int)(text.size()))
            {
                doneAnimating = true;
            }
        }
    }
    /// 20x32 font
    for(int i = 0; i < (int)(text.size()); i++)
    {
        if(text[i] == '\n')
        {
            line++;
            currCharacter = 0;
            continue;
        }
        unsigned spriteIndex = text[i] - '\'' + 13;
        sprite.setTextureRect(sf::IntRect((spriteIndex % 13) * 20,
                                          (int)(spriteIndex / 13) * 32, 20, 32));
        sf::Vector2f offset(x + charWidth * currCharacter * scale,
                           y + charHeight * line * scale);
        if(shake)
        {
            offset.x += (rand() / (float)RAND_MAX - 0.5f) * 2.f;
            offset.y += (rand() / (float)RAND_MAX - 0.5f) * 2.f;
        }
        if(wave)
        {
            offset.y += sinf((i + frame) / 5.f) * 2 * waveAmount;
        }
        if(animateIn)
        {
            if(animateChar < i)
            {
                break;
            }
            if(animateChar == i)
            {
                offset.y -= 100 * animateTimer / (float)animateSpeed;
            }
        }

        sprite.setPosition(offset);
        sprite.setScale(scale, scale);
        currCharacter++;
        window.draw(sprite);
    }
    waveAmount *= 0.96;
}
void EText::setColor(sf::Color color)
{
    sprite.setColor(color);
}

void EText::setString(string s)
{
    text = s;
}
