#include "Game.h"
#include "ResourceManager.h"
#include "Song.h"

const int Game::GAME_OVER_FRAMES = 120;
const int Game::GAME_WON_FRAMES = 120;
const int Game::GAME_WON_GRADE_FRAMES = 130;
const int Game::BEAT_SWITCH_FRAMES = 40;
const int Game::INTRO_FRAMES = 60;

Game::Game(sf::RenderWindow& window):
    window(window),
    view(window.getView()),
    gameState(STATE_INTRO),
    songIndex(0),
    song(nullptr),
    fx(sf::LinesStrip, Song::FFT_SAMPLES),
    leftPath({-60,0}, {300, 530}),
    rightPath({860, 0}, {500, 530}),
    frame(0),
    introDemon(0, 1, leftPath),
    leftTimer(0),
    rightTimer(0),
    titleSprite("resources/title.png", -1, -1, -1, 0,0, 0, false),
    demonSprite("resources/demons.png", 32, 32, -1, 0, 0, 0, false),
    headSprite("resources/head.png", 200, 200, -1, 0, 0, 0, false),
    headSadSprite("resources/head_sad.png", 200, 200, -1, 0, 0, 0, false),
    headphoneSprite("resources/headphones.png", 200, 200, -1, 0, 0, 0, false),
    headphoneLeftSprite("resources/headphones_left.png", 200, 200, -1, 0, 0, 0, false),
    headphoneRightSprite("resources/headphones_right.png", 200, 200, -1, 0, 0, 0, false),
    controlSprite("resources/controls.png", -1, -1, -1, 0, 0, 0, false),
    songNameSprite("resources/songNames.png", 128, 51, -1, 0, 0, 0, false),
    notesSprite("resources/notes.png", 32, 32, -1, 0, 0, 0, false),
    arrowSprite("resources/arrow.png", -1, -1, -1, 0,0,0, false)
{
    sf::Color lineColor = sf::Color(103, 153, 158);
    for(int i = 0; i < Song::FFT_SAMPLES;i++)
    {
        int distance = abs(i - (Song::FFT_SAMPLES / 2));
        lineColor.r = distance;
        fx[i].color = lineColor;
    }
    win = false;
    input.loadDefaults();
    rightPath.opposite = true;
    missSound.setBuffer(*ResourceManager::getSoundBuffer("resources/miss.ogg"));
    unlockedSongs = {true, false, false};
    newSongs = {true, true, true};
    topScores = {-1, -1, -1};

    sf::SoundBuffer* buffer = ResourceManager::getSoundBuffer("resources/song1.ogg");
    assert(buffer);
    faceColor = sf::Color(125, 141, 176);
    reset();
    printf("Game initialized.\n");
    introOffsets = {
        110, // pause
        160, //demon
        50, //bad feel
        120, //headphones fadein
        120, //headphones fall
        10

    };
}

Game::~Game()
{
    if(song)
    {
        delete song;
    }
    for(auto d_it = demons.begin(); d_it != demons.end(); d_it++)
    {
        delete d_it->second;
    }
    demons.clear();
}


void Game::step()
{
    frame++;
    input.step();
    if(leftTimer)
    {
        leftTimer--;
    }
    if(rightTimer)
    {
        rightTimer--;
    }
    if(missTimer)
    {
        missTimer--;
    }
    switch(gameState)
    {
    case STATE_INTRO:
        stepIntro();
        if(input.held(Input::LEFT))
        {
            stepIntro();
        }
        if(input.held
           (Input::RIGHT))
        {
            stepIntro();
        }
        break;
    case STATE_SONG:
        stepGame();
        break;
    case STATE_CHOOSE_LEVEL:
        stepMenu();
        break;
    case STATE_SCORE:
        stepScore();
        break;
    case STATE_CREDITS:
        stepCredits();
        break;
    default:
        assert(false);
        break;
    }
}
void Game::stepCredits()
{
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Escape) ||
       sf::Keyboard::isKeyPressed(sf::Keyboard::Q))
    {
        if(song)
        {
            song->stop();
        }
        setState(STATE_CHOOSE_LEVEL);
        return;
    }
    if(secret.open)
    {
        if(input.held(Input::DOWN))
        {
            secret.down();
        }
        if(input.held(Input::UP))
        {
            secret.up();
        }
        if(input.held(Input::LEFT))
        {
            secret.left();
        }
        if(input.held(Input::RIGHT))
        {
            secret.right();
        }
    }
    else
    {
        if(input.pressed(Input::DOWN))
        {
            secret.reset();
        }
    }
    secret.step();
}
void Game::stepScore()
{
    if(!win)
    {
        if(frame > 340)
        {
            setState(STATE_CHOOSE_LEVEL);
        }
    }
    else
    {
        if(frame > 440)
        {
            setState(STATE_CREDITS);
        }
    }
}
void Game::stepMenu()
{
    if(input.pressed(Input::LEFT))
    {
        if(songSelection > 0)
        {
            //TODO: select fx
            songSelection--;
        }
    }
    if(input.pressed(Input::RIGHT))
    {
        if(songSelection < unlockedSongs.size()-1 && unlockedSongs[songSelection+1])
        {
            songSelection++;
        }
    }
    if(input.pressed(Input::ACTION))
    {
        //Song chosen
        printf("songSelection = %d\n", songSelection);
        songIndex = songSelection;
        setState(STATE_SONG);
    }
}
void Game::stepIntro()
{
//
//    if(sf::Keyboard::isKeyPressed(sf::Keyboard::T))
//    {
//        setState(STATE_CREDITS);
//        return;
//    }
    introTimer++;
    introSubTimer++;

    int totalOffset = 0;
    for(int i = 0; i < (int)(introOffsets.size()); i++)
    {

        if(introTimer > introOffsets[i] + totalOffset)
        {
            totalOffset += introOffsets[i];
        }
        else if(introTimer == introOffsets[i] + totalOffset)
        {
            introSubTimer = 0;
            introSubState = i+1;
            if(introSubState == 2)
            {
                miss();
            }
            break;
        }
        else
        {
            introSubState = i;
            if(i == 0)
            {
                //pause
            }
            else if(i == 1)
            {
                //demon
            }
            else if(i == 2)
            {
                // miss effc
                missTimer = 2;
            }
            else if(i == 3)
            {
                //fade in hed
            }
            else if(i == 4)
            {
                //drop hed
            }
            else if(i == 5)
            {
                setState(STATE_CHOOSE_LEVEL);
            }
            break;
        }
    }

}
void Game::stepGame()
{
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Escape) ||
       sf::Keyboard::isKeyPressed(sf::Keyboard::Q))
    {
        if(song)
        {
            song->stop();
        }
        setState(STATE_CHOOSE_LEVEL);
        return;
    }
//    if(sf::Keyboard::isKeyPressed(sf::Keyboard::S))
//    {
//        //cheat code. disable this later.
//        setState(STATE_SCORE);
//        return;
//    }

    if(level.events.size() == 0)
    {
        songEndTimer++;
        if(songEndTimer == 100)
        {
            setState(STATE_SCORE);
        }
        return;
    }
    else
    {
        songEndTimer = 0;
    }
    if(input.pressed(Input::LEFT))
    {
        int offset = level.getOffsetToEvent(EVENT_KICK);
        int diff = offset - song->getOffset();
        if(abs(diff) < 10000)
        {
            leftTimer = 20;
            int index = level.popEvent(EVENT_KICK);
            auto dem_it = demons.find(index);
            if(dem_it != demons.end())
            {
                dem_it->second->hit();
            }
        }
        else
        {
            miss();
        }
    }
    if(input.pressed(Input::RIGHT))
    {
        int offset = level.getOffsetToEvent(EVENT_TAMB);

        int diff = offset - song->getOffset();
        if(abs(diff) < 10000)
        {
            rightTimer = 20;
            int index = level.popEvent(EVENT_TAMB);
            auto dem_it = demons.find(index);
            if(dem_it != demons.end())
            {
                dem_it->second->hit();
            }
        }
        else
        {
            miss();
        }

    }

    leftPath.step();
    rightPath.step();
    song->update();
    handleLevelEvents();
    if(ResourceManager::shader)
    {
        //ResourceManager::shader->setParameter("amplitude", amp);
    }
}

void Game::drawFace()
{
    float faceX = 400;
    if(missTimer > 0)
    {
        faceX += (rand()/(float)RAND_MAX - 0.5f) * 5;
    }
    if(leftTimer && rightTimer)
    {
    }
    else
    {
        faceX += rightTimer / 5.f - leftTimer / 5.f;
    }
    float targetFaceY = 520 + leftTimer / 3.f + rightTimer / 3.f;
    faceY += (targetFaceY-faceY)/ 1.5f;
    headphoneY += (faceY - headphoneY) / 3.f;
    if(missTimer > 0)
    {
        headSadSprite.setPosition(faceX, faceY);
        headSadSprite.draw(window);
    }
    else
    {
        headSprite.setPosition(faceX, faceY);
        headSprite.draw(window);
    }
    float realHeadPhonePos = headphoneY;
//    if(gameState == STATE_INTRO)
//    {
//        realHeadPhonePos -= (1 - (introTimer / 120.f)) * 100;
//    }
    if(gameState == STATE_INTRO)
    {
        float alpha = 1.f;
        float offset = 300;
        //4 - fade. 5-fall
        if(introSubState < 3)
        {
            alpha = 0;
        }
        else if(introSubState == 3)
        {
            alpha = introSubTimer / (float)introOffsets[3];
        }
        else if(introSubState == 4)
        {
            offset = 300 * powf(1 - introSubTimer / (float)introOffsets[4], 2);
        }
        else
        {
            offset = 0;
        }

        realHeadPhonePos -= offset;
        headphoneSprite.setColor(sf::Color(255, 255, 255, 255 * alpha));
    }
    else
    {
        headphoneSprite.setColor(sf::Color::White);
    }
    headphoneSprite.setPosition(faceX, realHeadPhonePos);
    headphoneSprite.draw(window);
    if(rightTimer > 0)
    {
        float amt = rightTimer / 20.f;
        headphoneRightSprite.setColor(sf::Color(255, 255, 255, 255 * amt));
        headphoneRightSprite.setPosition(faceX, headphoneY);
        headphoneRightSprite.draw(window);
    }
    if(leftTimer > 0)
    {
        float amt = leftTimer / 20.f;
        headphoneLeftSprite.setColor(sf::Color(255, 255, 255, 255 * amt));
        headphoneLeftSprite.setPosition(faceX, headphoneY);
        headphoneLeftSprite.draw(window);
    }
}
void Game::draw()
{

    window.clear(sf::Color(36 + rightTimer, 101 - leftTimer, 107));

    /// draw fft
    if(song)
    {
//        sf::RectangleShape line({20, 10});
//        line.setFillColor(sf::Color(16, 78, 84));
//        line.setOrigin(10, 0);

        float windowScale = 800 / ((float)Song::FFT_SAMPLES);
        for(unsigned i = 0; i < Song::FFT_SAMPLES; i++)
        {
            if(gameState == STATE_SONG)
            {

                float f = std::real(song->fftResult[i])/1000.f;
                if(fabs(f) > fabs(fftDrawAmount[i]))
                {
                    fftDrawAmount[i] = f;
                }
                else
                {
                    fftDrawAmount[i] *= 0.91f;
                }
            }
            else
            {
                fftDrawAmount[i] *= 0.9f;
            }
            fx[i].position = sf::Vector2f(i * windowScale, 530 - fftDrawAmount[i]);
           // printf("%d: %f\n", i, line.getSize().y);
//           line.setSize({20, fftDrawAmount[i]});
//            line.setPosition(800 * ((i * 2) / (float)Song::FFT_SAMPLES), 600 -fftDrawAmount[i]);
//
//            window.draw(line);
        }
        sf::Transform offset;

        window.draw(fx);
        offset.translate(1, 0);
        window.draw(fx, offset);
        offset.translate(-2, 0);
        window.draw(fx, offset);
        offset.translate(1, 1);
        window.draw(fx, offset);
        offset.translate(0, -2);
        window.draw(fx, offset);

    }
    static sf::Vector2f origin(400, 0);
    static sf::Vector2f target(400, 500);
    static int totalTimeAhead = 150000;
    drawFace();

    if(gameState == STATE_SONG)
    {
        int indexToDrawUpTo = level.getEventIndexAtTime(song->getOffset(), totalTimeAhead);

        if(indexToDrawUpTo < 0)
        {
            indexToDrawUpTo = 0;
        }
        auto dem_it = demons.begin();
        for(int i = indexToDrawUpTo; i < (int)(level.events.size()); i++)
        {
            float percent = 1 - (level.events[i].offset - song->getOffset()) / (float)totalTimeAhead;
            dem_it = demons.find(i);
            if(dem_it == demons.end())
            {
                if(level.events[i].type == EVENT_KICK)
                {
                    demons.insert(pair<int, Demon*>(i, new Demon(i, i % 6, leftPath)));
                }
                else if(level.events[i].type == EVENT_TAMB)
                {
                    demons.insert(pair<int, Demon*>(i, new Demon(i, i % 6 + 6, rightPath)));
                }
            }
            else
            {
                dem_it->second->setPercent(percent);
            }
        }

        for(dem_it = demons.begin(); dem_it != demons.end(); )
        {
            dem_it->second->step();
            dem_it->second->draw(window, 0, leftTimer + rightTimer);
            if(dem_it->second->lifeTimer <= 0)
            {
                delete dem_it->second;
                dem_it = demons.erase(dem_it);
            }
            else
            {
                dem_it++;
            }
        }
        if(frame < 60)
        {
            drawControls(frame / 60.f);
        }
        else if(frame < 300)
        {
            drawControls(1 - (frame - 60) / (float)(300-60));
        }

    }
    else if(gameState == STATE_INTRO)
    {
        if(introSubState == 1)
        {
            introDemon.setPercent(introSubTimer / 120.f);
            introDemon.step();
            introDemon.draw(window, 40);
        }
    }
    else if(gameState == STATE_CHOOSE_LEVEL)
    {
        drawMenu();
    }
    else if(gameState == STATE_SCORE)
    {
        drawScore();
    }
    else if(gameState == STATE_CREDITS)
    {
        drawCredits();
    }

    // black bars, when window is big.
    if(view.getSize().x > 800)
    {
        sf::Color clearColor(7 + rightTimer / 2.f, 22 + leftTimer / 2.f, 63 - missTimer / 2.f);
        sf::RectangleShape blackBar({(view.getSize().x - 800)/2.f, view.getSize().y});
        blackBar.setPosition(-blackBar.getSize().x, -(view.getSize().y - 600) / 2.f);
        blackBar.setFillColor(clearColor);

        window.draw(blackBar);
        blackBar.move(800 + blackBar.getSize().x, 0);

        window.draw(blackBar);

    }
}

void Game::drawScore()
{
    auto alphaText = [&](string s, float x, float y, int startFrame, int fadeFrames)
    {
        sf::Text text(s, *ResourceManager::font, 21);
        float alpha = 0;
        if(frame > startFrame + fadeFrames)
        {
            alpha = 1.f;
        }
        else if(frame >= startFrame)
        {
            alpha = (frame - startFrame) / (float)fadeFrames;
        }
        text.setColor(sf::Color(255, 226, 165, alpha * 255));
        text.setPosition(x, y);
        text.setRotation(sinf(frame / 30.f) * 1.f);
        window.draw(text);
    };

    float textX = 120;
    float textY = 130;
    if(passed)
    {
        alphaText("Nicely done!", textX, textY, 0, 30);
        stringstream missSs;
        if(misses == 0)
        {
            missSs << "You didn't miss a single demon!";
        }
        else
        {
            missSs << "You only missed " << misses;
            if(misses == 1)
            {
                missSs << " demon.";
            }
            else
            {
                missSs << " demons.";
            }
        }

        alphaText(missSs.str(), textX, textY + 30, 90, 30);
        if(unlockedNewSong)
        {
            alphaText("New Song Unlocked!", textX, textY + 60, 200, 30);
        }
    }
    else
    {
        stringstream missSs;
        missSs << "You missed " << misses << " demons!!!";
        alphaText(missSs.str(), textX, textY, 0, 30);
        alphaText("Can you do any better?", textX, textY + 30, 90, 30);
    }




}

void Game::drawCredits()
{
    sf::RectangleShape overlay({800, 600});
    float alpha = 1.f;
    if(frame < 120)
    {
        alpha = frame / 120.f;
    }
    overlay.setPosition(0, 0);
    overlay.setFillColor(sf::Color(0, 0, 0, alpha * 255));
    window.draw(overlay);

    sf::Text youDidIt("Congratulations!", *ResourceManager::font, 32);
    sf::Text thanks("Made in 48 hours for Ludum Dare 32", *ResourceManager::font, 20);
    sf::Text credits("by ekun (@ekunenuke)",*ResourceManager::font, 20);
    youDidIt.setPosition(145,100);
    youDidIt.setColor(sf::Color(255, 255, 255));
    window.draw(youDidIt);
    thanks.setPosition(145, 150);
    window.draw(thanks);
    credits.setPosition(145, 180);
    window.draw(credits);

    if(frame < 120)
    {
        return;
    }
    float demonAlpha = 1.f;
    if(frame < 200)
    {
        demonAlpha = (frame - 120) / 80.f;
    }
    demonSprite.setScale(sinf(frame/30.f) * 0.3 + 1);
    demonSprite.setColor(sf::Color(255, 255, 255, demonAlpha * 255));
    for(unsigned i = 0; i <12; i++)
    {
        float revX = fmodf(800 * i / 12.f - frame + 1243254, 800);
        demonSprite.setFrame(i);
        demonSprite.setPosition(revX, 360);
        demonSprite.draw(window);
        demonSprite.setPosition(fmodf(frame + 820 * i / 12.f, 820), 400);
        demonSprite.draw(window);
        demonSprite.setPosition(revX, 440);
        demonSprite.draw(window);
        demonSprite.setPosition(fmodf(frame + 820 * i / 12.f, 820), 480);
        demonSprite.draw(window);
        demonSprite.setPosition(revX, 520);
        demonSprite.draw(window);
    }
    secret.draw(window);
    if(!secret.open)
    {
        arrowSprite.setPosition(400, 570 + sinf(frame / 60.f) * 2);
        arrowSprite.draw(window);
    }
}
void Game::drawControls(float alpha)
{
    controlSprite.setColor(sf::Color(255, 255, 255, alpha * 255));
    controlSprite.setFlipped(false);
    controlSprite.setPosition(150, 500);
    controlSprite.draw(window);
    controlSprite.setFlipped(true);
    controlSprite.setPosition(650, 500);
    controlSprite.draw(window);
}

void Game::drawMenu()
{
    float alpha = 1.f;
    if(frame < 60)
    {
        alpha = frame / 60.f;
    }
    titleSprite.setColor(sf::Color(255, 255, 255, alpha * 255));
    titleSprite.setPosition(400, 200);
    titleSprite.draw(window);
    float x = 400;
    float y = 360;
    for(int i = 0; i < 3; i++)
    {
        int xOff = (i-1) * 170;
        songNameSprite.setPosition(x + xOff, y);
        songNameSprite.setFrame(i);
        if(i == (int)songSelection)
        {
            songNameSprite.setColor(sf::Color(255, 255, 255, alpha * 255));
            notesSprite.setFrame(0);
            notesSprite.setPosition(x + xOff, y - 70 + cosf(frame / 20.f) * 6.f);
            notesSprite.draw(window);
        }
        else
        {
            if(!unlockedSongs[i])
            {
                songNameSprite.setColor(sf::Color(10, 10, 10, alpha * 255));
            }
            else
            {
                songNameSprite.setColor(sf::Color(200, 200, 200, alpha * 255));
            }
        }
        if(i > 0 && unlockedSongs[i] && newSongs[i])
        {
            sf::Text newText("NEW", *ResourceManager::font, 17);
            newText.setPosition(x + xOff - 20, y + 30);
            newText.setColor(sf::Color(255, 255, 255, sinf(frame / 20.f) * 100 + 155));
            window.draw(newText);
        }
        if(topScores[i] >= 0)
        {
            stringstream ss;
            if(topScores[i] == 0)
            {
                ss << "PERFECT";
            }
            else
            {
                ss << "BEST:-" << topScores[i];
            }
            sf::Text bestText(ss.str(), *ResourceManager::font, 17);
            bestText.setPosition(x + xOff - 54, y + 30);
            bestText.setColor(sf::Color(200, 200, 200, 255));
            window.draw(bestText);
        }
        songNameSprite.draw(window);
    }
}

void Game::resized()
{
    view.setSize(window.getSize().x, window.getSize().y);
    window.setView(view);
}
void Game::miss()
{
    missTimer = 20;
    misses++;
    missSound.play();
    printf("MISS! %d\n", misses);
}

void Game::handleLevelEvents()
{
    while(true)
    {
        EventType type = level.pollEvent(song->getOffset(), 10000);
        if(type == NO_EVENT)
        {
            break;
        }
        auto dem_it = demons.find(level.events.size());
        if(dem_it != demons.end())
        {
            dem_it->second->miss();
            miss();
        }

    }
}

void Game::setState(GameState newState, bool force)
{
    if(gameState == newState && !force)
    {
        return;
    }
    frame = 0;
    missTimer = 0;
    faceY = 520;
    headphoneY = 520;
    gameState = newState;
    switch(gameState)
    {
    case STATE_INTRO:
        introTimer = 0;
        break;
    case STATE_SONG:
        if(song)
        {
            delete song;
            song = nullptr;
        }
        for(int i = 0; i < Song::FFT_SAMPLES; i++)
        {
            fftDrawAmount[i] = 0;
        }
        misses = 0;
        amp = 0;
        passed = false;

        level = Level();

        sf::SoundBuffer* buffer;


        if(songIndex == 0)
        {
            level.load("resources/lvl2.lvl");
            buffer = ResourceManager::getSoundBuffer("resources/song2.ogg");
        }
        else if(songIndex == 1)
        {
            level.load("resources/lvl1.lvl");
            buffer = ResourceManager::getSoundBuffer("resources/song1.ogg");
            //invalid index.
        }
        else if(songIndex == 2)
        {
            level.load("resources/lvl3.lvl");
            buffer = ResourceManager::getSoundBuffer("resources/song3.ogg");
        }

        song = new Song(buffer);

        song->play();


        for(auto d_it = demons.begin(); d_it != demons.end(); d_it++)
        {
            delete d_it->second;
        }
        demons.clear();
        break;
    case STATE_SCORE:
        newSongs[songIndex] = false;
        if(misses < topScores[songIndex] || topScores[songIndex] < 0)
        {
            topScores[songIndex] = misses;
        }

        unlockedNewSong = false;
        //evaluate whether we passed or not
        passed = misses < 20;
        if(passed)
        {
            if(songIndex < unlockedSongs.size()-1)
            {
                if(!unlockedSongs[songIndex+1])
                {
                    unlockedNewSong = true;
                    unlockedSongs[songIndex+1] = true;
                }
            }
            else if(songIndex == unlockedSongs.size()-1)
            {
                win = true;
            }
        }

        break;
    case STATE_CHOOSE_LEVEL:
        ResourceManager::creditsMusic->stop();
        break;
    case STATE_CREDITS:
        if(song)
        {
            delete song;
            song = nullptr;
        }
        if(secret.open)
        {
            secret.gameOver();
        }
        ResourceManager::creditsMusic->play();
        break;
    default:
        assert(false);
        break;
    }
}
void Game::reset(bool smoothReset)
{
    unlockedNewSong = false;
    songSelection = 0;
    songIndex = 0;
    setState(STATE_INTRO, true);
}
