#ifndef GAME_H
#define GAME_H

#include "Input.h"
#include "Level.h"
#include "EText.h"
#include "Path.h"

#include "Sprite.h"
#include "Demon.h"

#include "Song.h"
#include "Secret.h"


class Game
{
    private:
        enum GameState
        {
            STATE_INTRO,
            STATE_SONG,
            STATE_SCORE,
            STATE_CHOOSE_LEVEL,
            STATE_CREDITS,
            NUM_GAME_STATES
        };
        enum EffectType
        {

            NUM_EFFECTS
        };
        static const int GAME_OVER_FRAMES;
        static const int GAME_WON_FRAMES;
        static const int GAME_WON_GRADE_FRAMES;
        static const int INTRO_FRAMES;
        static const int BEAT_SWITCH_FRAMES;


    public:
        Game(sf::RenderWindow& window);
        ~Game();
        void stepBind();
        void step();
        void stepIntro();
        void stepGame();
        void stepMenu();
        void stepScore();
        void stepCredits();
        void drawFace();


        void draw();
        void drawControls(float alpha);
        void drawMenu();
        void drawScore();
        void drawCredits();
        void resized();
    private:
        void miss();
        void handleLevelEvents();
        void setState(GameState newState, bool force = false);
        void reset(bool smoothReset = false);


        sf::RenderWindow& window;
        sf::View view;
        GameState gameState;
        Input input;
        Level level;
        sf::Sound missSound;
        unsigned songSelection;
        unsigned songIndex;
        vector<bool> unlockedSongs;
        vector<bool> newSongs;
        vector<int> topScores;
        Song* song;

        sf::VertexArray fx;

        Path leftPath;
        Path rightPath;

        unsigned frame;

        map<int, Demon*> demons;


        vector<int> introOffsets;
        int introTimer;
        int introSubState;
        int introSubTimer;
        int songEndTimer;
        Demon introDemon;

        int leftTimer;
        int rightTimer;
        int missTimer;
        sf::Color faceColor;

        Sprite titleSprite;
        Sprite demonSprite;
        Sprite headSprite;
        Sprite headSadSprite;
        Sprite headphoneSprite;
        Sprite headphoneLeftSprite;
        Sprite headphoneRightSprite;
        Sprite controlSprite;
        Sprite songNameSprite;
        Sprite notesSprite;
        Sprite arrowSprite;
        float headphoneY;
        float faceY;

        float fftDrawAmount[Song::FFT_SAMPLES];

        int misses;
        bool passed;
        bool win;
        bool unlockedNewSong;
        int hits;
        float amp;

        // credits vars
        Secret secret;
};

#endif // GAME_H
