#include "Input.h"

Input::Input():
    joystickId(0),
    gamepadDetected(false),
    usingGamepad(false),
    fullyBound(false),
    currentlyBinding(LEFT),
    bindBeatCounter(0)
{
    prevKeys.resize(sf::Keyboard::KeyCount, false);
    keys.resize(sf::Keyboard::KeyCount, false);

    actionKeyBindings.resize(NUM_INPUT_ACTIONS, sf::Keyboard::F1);
    actionGamepadBindings.resize(NUM_INPUT_ACTIONS, {});

    actionStates.resize(NUM_INPUT_ACTIONS, false);
    prevActionStates.resize(NUM_INPUT_ACTIONS, false);
}

void Input::step()
{
//    for(unsigned i = 0; i < sf::Joystick::Count; i++)
//    {
//        if(!sf::Joystick::isConnected(i))
//        {
//            continue;
//        }
//        for(unsigned j = 0; j < sf::Joystick::AxisCount; j++)
//        {
//            if(!sf::Joystick::hasAxis(i, (sf::Joystick::Axis)j))
//            {
//                continue;
//            }
//            printf("Joystick %d, Axis %d, %f\n", i, j, sf::Joystick::getAxisPosition(i, (sf::Joystick::Axis)j));
//        }
//    }
//    printf("\n");
    for(unsigned i = 0; i < sf::Keyboard::KeyCount; i++)
    {
        prevKeys[i] = keys[i];
        keys[i] = sf::Keyboard::isKeyPressed((sf::Keyboard::Key)i);
    }

    for(unsigned i = 0; i < NUM_INPUT_ACTIONS; i++)
    {
        prevActionStates[i] = actionStates[i];
        if(!usingGamepad)
        {
            actionStates[i] = sf::Keyboard::isKeyPressed(actionKeyBindings[i]);
        }
        else
        {
            actionStates[i] = isGamepadActionPressed(joystickId,
                                                     actionGamepadBindings[i].axis,
                                                     actionGamepadBindings[i].button,
                                                     actionGamepadBindings[i].axisId,
                                                     actionGamepadBindings[i].axisDir);

//            if(actionGamepadBindings[i].axis)
//            {
//                printf("\nAxis ID: %d\n Axis Dir: %d\n", actionGamepadBindings[i].axisId, actionGamepadBindings[i].axisDir);
//                printf("j[%d] %d\n", joystickId, (actionStates[i] ? 1 : 0));
//                printf("%f\n", sf::Joystick::getAxisPosition(joystickId, (sf::Joystick::Axis)actionGamepadBindings[i].axisId));
//            }
        }
    }
}

void Input::updateBind(InputAction action, int beatScore)
{
    bool controllerSelected = (action != LEFT);

    if(usingGamepad || !controllerSelected)
    {
        gamepadDetected = false;
        vector<unsigned> connectedIds;
        for(unsigned i = 0; i < sf::Joystick::Count; i++)
        {
            if(!sf::Joystick::isConnected(i))
            {
                continue;
            }
            gamepadDetected = true;
            connectedIds.push_back(i);
        }
        if(gamepadDetected)
        {
            unsigned chosenId = 0;
            unsigned chosenButton = 0;
            unsigned chosenAxis = 0;
            bool choseToUseAxis = false;
            int chosenAxisDirection = 0;

            int numActionsDetected = 0;

            for(unsigned i = 0; i < connectedIds.size(); i++)
            {
                unsigned id = connectedIds[i];
                for(unsigned j = 0; j < sf::Joystick::ButtonCount; j++)
                {
                    if(sf::Joystick::isButtonPressed(id, j))
                    {
                        chosenId = id;
                        chosenButton = j;
                        choseToUseAxis = false;
                        numActionsDetected++;
                    }
                }
                for(unsigned j = 0; j < sf::Joystick::AxisCount; j++)
                {
                    if(!sf::Joystick::hasAxis(id, (sf::Joystick::Axis)(j)))
                    {
                        continue;
                    }
                    if(isGamepadActionPressed(id, true, 0, j, 1))
                    {
                        chosenId = id;
                        numActionsDetected++;
                        choseToUseAxis = true;
                        chosenAxis = j;
                        chosenAxisDirection = 1;
                    }

                    if(isGamepadActionPressed(id, true, 0, j, -1))
                    {
                        chosenId = id;
                        numActionsDetected++;
                        choseToUseAxis = true;
                        chosenAxis = j;
                        chosenAxisDirection = -1;
                    }
                }
            }

            if(numActionsDetected == 1)
            {
                GamepadAction action;
                action.axis = choseToUseAxis;
                action.axisId = chosenAxis;
                action.button = chosenButton;
                action.axisDir = chosenAxisDirection;
                unsigned prevJoystickId = joystickId;
                joystickId = chosenId;


                /// check if it's the same button/axis, and same controller
                if(actionGamepadBindings[currentlyBinding].axis != action.axis ||
                   actionGamepadBindings[currentlyBinding].axisId != action.axisId ||
                   actionGamepadBindings[currentlyBinding].button != action.button ||
                   actionGamepadBindings[currentlyBinding].axisDir != action.axisDir ||
                   joystickId != prevJoystickId
                   )
                {
                    bindBeatCounter = 0;
                }

                /// make sure it wasn't already used
                bool actionAlreadyUsed = false;
                for(unsigned i = 0; i < currentlyBinding; i++)
                {
                    if(actionGamepadBindings[i].axis == action.axis &&
                       actionGamepadBindings[i].axisId == action.axisId &&
                       actionGamepadBindings[i].button == action.button &&
                       actionGamepadBindings[i].axisDir == action.axisDir)
                   {
                        actionAlreadyUsed = true;
                    }
                }

                if(!actionAlreadyUsed)
                {
//                    printf("Not already used! Diving deeper.\n");
                    actionGamepadBindings[currentlyBinding] = action;
                    usingGamepad = true;

                    if(action.axis)
                    {
//                        printf("\nAxis ID: %d\n Axis Dir: %d\n", action.axisId, action.axisDir);
                    }
                    if(pressed(currentlyBinding))
                    {
//                        printf("Pressed detected!\n");
                        if(beatScore < 2)
                        {
                            ///Reset binding of this key
                            bindBeatCounter = 0;
                        }
                        else
                        {
                            bindBeatCounter++;
                        }

                        /// Press key 3x in order to bind it.
                        if(bindBeatCounter >= 3)
                        {
                            currentlyBinding = (InputAction)(currentlyBinding + 1);
                        }
                        if(currentlyBinding == NUM_INPUT_ACTIONS)
                        {
                            fullyBound = true;
                        }
                    }
                }
            }
        }
    }

    if(!usingGamepad || !controllerSelected)
    {
        int numKeysPressed = 0;
        sf::Keyboard::Key key;
        for(unsigned i = 0; i < keys.size(); i++)
        {
            if(keys[i])
            {
                key = (sf::Keyboard::Key)i;
                numKeysPressed++;
            }
        }

        if(numKeysPressed == 1)
        {
            if(actionKeyBindings[currentlyBinding] != key)
            {
                bindBeatCounter = 0;
            }

            bool keyAlreadyUsed = false;
            for(unsigned i = 0; i < currentlyBinding; i++)
            {
                if(actionKeyBindings[i] == key)
                {
                    keyAlreadyUsed = true;
                }
            }

            if(!keyAlreadyUsed)
            {
                actionKeyBindings[currentlyBinding] = key;

                usingGamepad = false;
                if(pressed(currentlyBinding))
                {
                    if(beatScore < 2)
                    {
                        ///Reset binding of this key
                        bindBeatCounter = 0;
                    }
                    else
                    {
                        bindBeatCounter++;
                    }

                    /// Press key 3x in order to bind it.
                    if(bindBeatCounter >= 3)
                    {
                        currentlyBinding = (InputAction)(currentlyBinding + 1);
                    }
                    if(currentlyBinding == NUM_INPUT_ACTIONS)
                    {
                        fullyBound = true;
                    }
                }
            }
        }
    }

}

Input::InputAction Input::getActionCurrentlyBinding()
{
    return currentlyBinding;
}

void Input::loadDefaults()
{
    fullyBound = true;
    actionKeyBindings[LEFT] = sf::Keyboard::Left;
    actionKeyBindings[RIGHT] = sf::Keyboard::Right;
    actionKeyBindings[DOWN] = sf::Keyboard::Down;
    actionKeyBindings[UP] = sf::Keyboard::Up;
    actionKeyBindings[ACTION] = sf::Keyboard::Return;
    currentlyBinding = NUM_INPUT_ACTIONS;
}
bool Input::pressed(InputAction action)
{
    return actionStates[action] && !prevActionStates[action];
}

bool Input::released(InputAction action)
{
    return !actionStates[action] && prevActionStates[action];
}

bool Input::held(InputAction action)
{
    return actionStates[action];
}

bool Input::keyPressed(sf::Keyboard::Key key)
{
    return keys[key] && !prevKeys[key];
}
bool Input::keyReleased(sf::Keyboard::Key key)
{
    return !keys[key] && prevKeys[key];
}
bool Input::keyHeld(sf::Keyboard::Key key)
{
    return keys[key];
}

bool Input::isGamepadActionPressed(unsigned joystickId, bool axis, unsigned buttonId, unsigned axisId, int axisDir)
{
    if(axis)
    {
        if(axisDir > 0)
        {
//            printf("\t\t\tj[%d] %d, %d, %f\n", joystickId, axisId, axisDir, sf::Joystick::getAxisPosition(joystickId, (sf::Joystick::Axis)axisId));
            return sf::Joystick::getAxisPosition(joystickId, (sf::Joystick::Axis)axisId) > 50;
        }
        else
        {
            return sf::Joystick::getAxisPosition(joystickId, (sf::Joystick::Axis)axisId) < -50;
        }

    }
    else
    {
        return sf::Joystick::isButtonPressed(joystickId, buttonId);
    }
    return false;
}
