#ifndef INPUT_H
#define INPUT_H

#include "includes.h"

class Input
{
    public:
        enum InputAction
        {
            LEFT,
            RIGHT,
            UP,
            DOWN,
            ACTION,
            NUM_INPUT_ACTIONS
        };

        Input();

        void step();


        void updateBind(InputAction action, int beatScore);
        InputAction getActionCurrentlyBinding();


        void loadDefaults();

        bool pressed(InputAction action);
        bool released(InputAction action);
        bool held(InputAction action);


        bool keyPressed(sf::Keyboard::Key key);
        bool keyReleased(sf::Keyboard::Key key);
        bool keyHeld(sf::Keyboard::Key key);

        bool isGamepadActionPressed(unsigned joystickId, bool axis, unsigned buttonId, unsigned axisId, int axisDir);

    private:
        struct GamepadAction
        {
            bool axis;
            unsigned button;
            unsigned axisId;
            int axisDir;
        };

        vector<bool> prevKeys;
        vector<bool> keys;

        vector<sf::Keyboard::Key> actionKeyBindings;
        vector<GamepadAction> actionGamepadBindings;

        vector<bool> actionStates;
        vector<bool> prevActionStates;



    public:
        unsigned joystickId;

        bool gamepadDetected;
        bool usingGamepad;
        bool fullyBound;
        InputAction currentlyBinding;
        int bindBeatCounter;

};

#endif // INPUT_H
