#include "Level.h"

Level::Level()
{
    //ctor
}

Level::~Level()
{
    //dtor
}


void Level::load(string fileName)
{
    ifstream file(fileName);
    if(!file.is_open())
    {
        printf("ERROR: Unable to open level file!\n");
        return;
    }

    string line;
    unsigned measure = 0;
    bool gettingLineNumber = true;
    int measureStart;
    int measureEnd;
    while(getline(file, line))
    {
        line = trim(line);
        if(line.size() == 0)
        {
            continue;
        }
        if(gettingLineNumber)
        {
            if(line.find("-") != string::npos)
            {
                //We have a range of measures.
                size_t position = line.find("-");
                measureStart = atoi(line.substr(0, position).c_str());
                measureEnd = atoi(line.substr(position+1).c_str());
            }
            else
            {
                //We have a single measure
                 measureStart = atoi(line.c_str());
                 measureEnd = measureStart;
            }
            printf("%d-%d\n", measureStart, measureEnd);
        }
        else
        {
            addEvent(measureStart, measureEnd, line);
        }
        gettingLineNumber = !gettingLineNumber;
    }
    SortEvent sortEvent;
    sort(events.begin(), events.end(), sortEvent);
    reverse(events.begin(), events.end());
    file.close();
}
int Level::getOffsetToEvent(EventType type)
{
    for(int i = events.size()-1; i >= 0; i--)
    {
        if(events[i].type == type)
        {
            return events[i].offset;
        }
    }
    return -1;
}
int Level::popEvent(EventType type)
{
    for(int i = events.size()-1; i >= 0; i--)
    {
        if(events[i].type == type)
        {
            events[i].type = NO_EVENT;
            //events.erase(events.begin() + i);
            return i;

        }
    }
    printf("popEvent: no match found\n");
}
EventType Level::pollEvent(int offset, int keepAlive)
{
    while(events.size() > 0 && events[events.size()-1].type == NO_EVENT)
    {
        events.pop_back();
    }
    if(events.size() == 0 || (events[events.size()-1].offset + keepAlive)> offset)
    {
        return NO_EVENT;
    }

    EventType toReturn = events[events.size()-1].type;
    events.pop_back();
    return toReturn;

}


void Level::addEvent(int start, int end, string line)
{
    //start/end are 1 indexed
    start--;
    end--;

   // unsigned offset = 18900.f * measure * 2;
   // unsigned quarterOffset = 18900.f * ((float)measure + 0.25f) * 2.f;
    //unsigned halfOffset = 18900.f * ((float)measure + 0.5f) * 2.f;
    //unsigned threeQuarterOffset = 18900.f * ((float)measure + 0.75f) * 2.f;

    auto addEvent = [&](EventType event, int index)
    {
        for(int i = start; i <= end; i++)
        {
            int offset = 18900.f * (i + ((float)index / 16.f)) * 8.f; // need *2?
            events.push_back({offset, event});
        }
    };
    int index = 0;
    for(unsigned i = 0; i < line.size(); i++)
    {
        if(line[i] == '0')
        {

        }
        else if(line[i] == '1')
        {
            addEvent(EVENT_TAMB, index);
        }
        else if(line[i] == '2')
        {
            addEvent(EVENT_KICK, index);
        }
        else if(line[i] == '3')
        {
            addEvent(EVENT_KICK, index);
            addEvent(EVENT_TAMB, index);
        }
        else
        {
            continue;
        }
        index++;
    }

    assert(index == 16);


    /// convert string to lowercase
    //c = tolower(c);

}

void Level::drawNextEvents(sf::RenderWindow& window, unsigned offset)
{
    static const int LOOKAHEAD_AMOUNT = 200000;
    for(int i = events.size()-1; i >= 0; i--)
    {
        if(events[i].offset > offset + LOOKAHEAD_AMOUNT)
        {
            return;
        }
        int delta = events[i].offset - offset;
        sf::RectangleShape box({2, 5});
        switch(events[i].type)
        {
            default:
                break;
        }

        box.setPosition(delta / 380.f, 595);
        window.draw(box);
    }
}

int Level::getEventIndexAtTime(unsigned offset, int time)
{
     for(int i = events.size()-1; i >= 0; i--)
    {
        if(events[i].offset > offset + time)
        {
            return i;
        }
    }
    return -1;
}
