#include "ResourceManager.h"

map<string, sf::SoundBuffer*> ResourceManager::soundBuffers;
map<string, sf::Texture*> ResourceManager::textures;
sf::Shader* ResourceManager::shader = nullptr;
sf::Font* ResourceManager::font = nullptr;
sf::Music* ResourceManager::creditsMusic = nullptr;

bool ResourceManager::loadTexture(string filename)
{
    sf::Texture* texture = new sf::Texture();
    if(texture->loadFromFile(filename))
    {
        textures.insert(pair<string, sf::Texture*>(filename, texture));
        return true;
    }
    return false;
}
sf::Texture* ResourceManager::getTexture(string filename)
{
    auto tex_it = textures.find(filename);
    if(tex_it == textures.end())
    {
        return nullptr;
    }
    return tex_it->second;
}


bool ResourceManager::loadSoundBuffer(string filename)
{
    sf::SoundBuffer* buffer = new sf::SoundBuffer();
    if(buffer->loadFromFile(filename))
    {
        soundBuffers.insert(pair<string, sf::SoundBuffer*>(filename, buffer));
        return true;
    }
    return false;
}


sf::SoundBuffer* ResourceManager::getSoundBuffer(string filename)
{
    auto snd_it = soundBuffers.find(filename);
    if(snd_it == soundBuffers.end())
    {
        return nullptr;
    }
    return snd_it->second;
}

void ResourceManager::cleanUp()
{
    for(auto snd_it = soundBuffers.begin(); snd_it != soundBuffers.end(); snd_it++)
    {
        delete snd_it->second;
    }
    soundBuffers.clear();
    for(auto tex_it = textures.begin(); tex_it != textures.end(); tex_it++)
    {
        delete tex_it->second;
    }
    textures.clear();
    if(shader)
    {
        delete shader;
        shader = nullptr;
    }
    if(creditsMusic)
    {
        delete creditsMusic;
        creditsMusic = nullptr;
    }
}
