#include "Secret.h"
#include "ResourceManager.h"

Secret::Secret()
{
    open = false;
    highScore = 0;
    playerAccel =0.014;
    origin = {300, 700};
    targetOrigin = 700;
}

Secret::~Secret()
{
    //dtor
}

void Secret::down()
{
    if(player.dy < 2)
    {
        player.dy += playerAccel;
    }
}
void Secret::up()
{
    if(player.dy > -2)
    {
        player.dy -= playerAccel;
    }
}
void Secret::left()
{
    if(player.dx > -2)
    {
        player.dx -= playerAccel;
    }
}
void Secret::right()
{
    if(player.dx < 2)
    {
        player.dx += playerAccel;
    }
}
void Secret::reset()
{
    open = true;
    player.angle = 0;
    obstacles.clear();
    obstacles.resize(100, Obstacle());
    player.x = 100;
    player.y = 100;
    player.dx = 0;
    player.dy = 0;
    score = 0;
    frame = 0;
    difficulty = 3;
    spawnTimer = 0;
    targetOrigin = 300;
}

void Secret::gameOver()
{
    if(score > highScore)
    {
        highScore = score;
    }
    open = false;
    targetOrigin = 700;
}
void Secret::step()
{
    origin.y += (targetOrigin - origin.y) / 50.f;
    if(!open)
    {
        return;
    }
    frame++;
    if(difficulty < 100)
    {
        if(frame % (200+difficulty) == 0)
        {
            difficulty++;
            frame = 0;
        }
    }
    else
    {
        if(frame % 100)
        {
            score++;
        }
    }
    score = difficulty;
    int count = 0;
    int firstInactive = 0;
    for(unsigned i = 0; i < obstacles.size(); i++)
    {
        if(!obstacles[i].active)
        {
            firstInactive = i;
            continue;
        }
        obstacles[i].x += obstacles[i].dx;
        obstacles[i].y += obstacles[i].dy;
        if(obstacles[i].dx > 0 && obstacles[i].x > 200)
        {
            obstacles[i].active = false;
        }
        if(obstacles[i].dy > 0 && obstacles[i].y > 200)
        {
            obstacles[i].active = false;
        }
        if(obstacles[i].dx < 0 && obstacles[i].x < 0)
        {
            obstacles[i].active = false;
        }
        if(obstacles[i].dy < 0 && obstacles[i].dy < 0)
        {
            obstacles[i].active = false;
        }
        obstacles[i].angle += 1.2;
        count++;
    }
    if(count < difficulty)
    {
        Obstacle& o = obstacles[firstInactive];
        o.active = true;
        float angle;

        angle = PI / 2.f;
        o.x = rand()% 200;
        o.y = 0;

        float randomAngleBias = ((rand()/(float)RAND_MAX)-0.5f) * PI / 4.f;
        o.dx = cosf(angle + randomAngleBias) * 0.33;
        o.dy = sinf(angle + randomAngleBias) * 0.33;
        o.angle = rand()%360;
    }
    player.x += player.dx;
    player.y += player.dy;
    player.dx *= 0.99;
    player.dy *= 0.99;
    if(player.x < 0)
    {
        player.x = 0;
        if(player.dx < 0)
        {
            player.dx = -player.dx;
        }
    }
    if(player.x > 200)
   {
       player.x = 200;
       if(player.dx > 0)
       {
           player.dx = -player.dx;
       }
   }
   if(player.y < 0)
   {
       player.y = 0;
       if(player.dy < 0)
       {
           player.dy = -player.dy;
       }
   }
   if(player.y > 200)
   {
       player.y = 200;
       if(player.dy > 0)
       {
           player.dy = -player.dy;
       }
   }
   player.angle -= 1;

    for(unsigned i = 0; i < obstacles.size(); i++)
    {
        if(obstacles[i].active)
        {
            float distSq = powf(player.x - obstacles[i].x, 2) + powf(player.y - obstacles[i].y, 2);
            if(distSq < 50)
            {
                //collision
                gameOver();
                break;
            }
        }
    }
}

void Secret::draw(sf::RenderWindow& window)
{
    sf::RectangleShape bg({200, 200});
    bg.setPosition(origin);
    bg.setFillColor(sf::Color::Black);
    bg.setOutlineColor(sf::Color::White);
    bg.setOutlineThickness(5.f);
    window.draw(bg);

    sf::RectangleShape enemyShape({10, 10});
    enemyShape.setFillColor(sf::Color::Red);
    enemyShape.setOrigin(5, 5);
    for(unsigned i = 0; i < obstacles.size(); i++)
    {
        if(!obstacles[i].active)
        {
            continue;
        }
        enemyShape.setRotation(obstacles[i].angle);
        enemyShape.setPosition(origin.x + obstacles[i].x, origin.y + obstacles[i].y);
        window.draw(enemyShape);
    }

    sf::RectangleShape playerShape({7, 7});
    playerShape.setFillColor(sf::Color::White);
    playerShape.setOrigin(3.5, 3.5);
    playerShape.setPosition(origin.x + player.x, origin.y + player.y);
    playerShape.setRotation(player.angle);
    window.draw(playerShape);

    bg.setFillColor(sf::Color::Transparent);
    window.draw(bg);

    stringstream ss;
    ss<<"SCORE: ";
    ss << score;
    if(score < highScore)
    {
        ss << "   HIGHSCORE: "<<highScore;
    }
    sf::Text scoreText(ss.str(), *ResourceManager::font, 12);
    scoreText.setPosition(origin.x + 8, origin.y+8);
    scoreText.setColor(sf::Color::White);
    window.draw(scoreText);
}

