#ifndef SPRITE_H
#define SPRITE_H

#include "ResourceManager.h"

class Sprite
{
    public:
        Sprite(string textureName, int width, int height, int frameCount, int speed, float offsetX, float offsetY, bool loop);

        void step(const sf::Vector2i& frameRectDelta = sf::Vector2i(0,0), const sf::Vector2f& frameRectSizeMod = sf::Vector2f(1,1));
        void draw(sf::RenderTarget& window, sf::RenderStates states = sf::RenderStates::Default);

        void setFlipped(bool f);
        void setFrozen(bool f);
        void setReverse(bool r);
        bool getReverse();
        void setOrigin(float x, float y);
        void setOffset(float x, float y);
        void setPosition(float x, float y);
        void setPosition(const sf::Vector2f &pos);
        void setPositionNoOffset(float x, float y);
        void setRotation(float theta);
        void setFrame(int f);
        void setScale(float sx, float sy);
        void setScale(float s);
        void setColor(sf::Color color);
        float percentComplete();
        int getFrame();
        int getTotalFrames();
        int getNumFrames();
        int getTime();
        int getSpeed();
        float getRotation();
        void setSpeed(int s);
        sf::Vector2i getSize();
        sf::Vector2f getPosition();
        sf::Vector2f getOrigin();
        void reset();
        bool isFinished();
        void setSmooth(bool val);
        void rotate(float degrees);

        void setDone(bool val = true);
    public:
        bool flipped;
    private:
        sf::Sprite sprite;
        sf::IntRect frameContainer;
        int w;
        int h;
        int frame;
        int numFrames;
        int timer;
        int speed;
        int resetDelay;
        float angle;
        float scaleX;
        float scaleY;
        sf::Vector2f offset;
        bool reverse;
        bool frozen;
        bool loop;
        bool done;
        int rows;
        int cols;
};

#endif // SPRITE_H
