#include "Game.h"
#include "ResourceManager.h"
#include <ctime>

int main()
{
    sf::ContextSettings settings;
    settings.antialiasingLevel = 8;
    sf::RenderWindow window(sf::VideoMode(800, 600), "I dont want to hear it", sf::Style::Default, settings);
    window.setFramerateLimit(99);
    srand(time(0));

    ResourceManager::font = new sf::Font();
    ResourceManager::font->loadFromFile("resources/mingliu.ttc");

    auto loadingDots = [&](int numDots)
    {
        stringstream text;
        text << "LOADING";
        for(int i = 0; i < numDots; i++)
        {
            text << ".";
        }
        window.clear(sf::Color::Black);
        sf::Text loadingText(text.str(), *ResourceManager::font, 40);
        loadingText.setColor(sf::Color::White);
        loadingText.setOrigin(loadingText.getLocalBounds().width/2.f, loadingText.getLocalBounds().height/2.f);
        loadingText.setPosition(420, 250);
        window.draw(loadingText);
        window.display();
    };
    loadingDots(0);

    ResourceManager::loadSoundBuffer("resources/song1.ogg");
    loadingDots(1);
    ResourceManager::loadSoundBuffer("resources/song2.ogg");
    loadingDots(2);
    ResourceManager::loadSoundBuffer("resources/song3.ogg");
    loadingDots(3);
    ResourceManager::creditsMusic = new sf::Music();
    ResourceManager::creditsMusic->openFromFile("resources/credits.ogg");
    ResourceManager::creditsMusic->setLoop(true);

    loadingDots(0);
    ResourceManager::loadSoundBuffer("resources/miss.ogg");
    loadingDots(1);

    ResourceManager::loadTexture("resources/demons.png");
    loadingDots(2);
    ResourceManager::loadTexture("resources/head.png");
    loadingDots(3);
    ResourceManager::loadTexture("resources/head_sad.png");
    loadingDots(1);
    ResourceManager::loadTexture("resources/headphones.png");
    loadingDots(2);
    ResourceManager::loadTexture("resources/headphones_right.png");
    loadingDots(3);
    ResourceManager::loadTexture("resources/headphones_left.png");
    loadingDots(1);
    ResourceManager::loadTexture("resources/notes.png");
    loadingDots(2);
    ResourceManager::loadTexture("resources/controls.png");
    loadingDots(3);
    ResourceManager::loadTexture("resources/songNames.png");
    loadingDots(1);
    ResourceManager::loadTexture("resources/title.png");
    ResourceManager::loadTexture("resources/arrow.png");
    loadingDots(2);

    if(sf::Shader::isAvailable())
    {
        ResourceManager::shader = new sf::Shader();


        if(!ResourceManager::shader->loadFromFile("resources/shader.frag", sf::Shader::Fragment))
        {
            delete ResourceManager::shader;
            ResourceManager::shader = nullptr;
        }
    }

    Game game(window);

    while(window.isOpen())
    {
        sf::Event event;
        while(window.pollEvent(event))
        {
            if (event.type == sf::Event::Closed)
            {
                window.close();
            }
            else if(event.type == sf::Event::Resized)
            {
                game.resized();
            }
        }

        game.step();
        game.draw();

        window.display();
    }
    ResourceManager::cleanUp();
    return 0;
}
